=== modified file 'bzrlib/_walkdirs_win32.pyx'
--- bzrlib/_walkdirs_win32.pyx	2011-04-04 11:24:50 +0000
+++ bzrlib/_walkdirs_win32.pyx	2011-04-05 12:15:34 +0000
@@ -174,6 +174,7 @@
 
     def top_prefix_to_starting_dir(self, top, prefix=""):
         """See DirReader.top_prefix_to_starting_dir."""
+        global osutils
         if osutils is None:
             from bzrlib import osutils
         return (osutils.safe_utf8(prefix), None, None, None,

=== modified file 'bzrlib/crash.py'
--- bzrlib/crash.py	2011-04-04 13:37:25 +0000
+++ bzrlib/crash.py	2011-04-05 12:23:03 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010, 2011 Canonical Ltd
+# Copyright (C) 2009-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/per_repository/test_has_same_location.py'
--- bzrlib/tests/per_repository/test_has_same_location.py	2011-03-11 21:06:09 +0000
+++ bzrlib/tests/per_repository/test_has_same_location.py	2011-04-02 22:51:52 +0000
@@ -20,6 +20,9 @@
     bzrdir,
     transport,
     )
+from bzrlib.remote import (
+    RemoteRepositoryFormat,
+    )
 from bzrlib.tests import (
     TestNotApplicable,
     )
@@ -84,21 +87,19 @@
         CopyConverter creates a second repository in one bzrdir.
         """
         repo = self.make_repository('repo')
-        try:
-            control_transport = repo._transport
-        except AttributeError:
-            raise TestNotApplicable(
-                "%r has no transport" % (repo,))
-        if control_transport.base == repo.bzrdir.transport.base:
+        if repo.control_transport.base == repo.bzrdir.control_transport.base:
             raise TestNotApplicable(
                 "%r has repository files directly in the bzrdir"
                 % (repo,))
             # This test only applies to repository formats where the repo
             # control_files are separate from other bzrdir files, i.e. metadir
             # formats.
-        control_transport.copy_tree('.', '../repository.backup')
-        backup_transport = control_transport.clone('../repository.backup')
-        backup_repo = repo._format.open(repo.bzrdir, _found=True,
+        repo.control_transport.copy_tree('.', '../repository.backup')
+        backup_transport = repo.control_transport.clone('../repository.backup')
+        if isinstance(repo._format, RemoteRepositoryFormat):
+            raise TestNotApplicable("remote repositories don't support overriding "
+                                    "transport")
+        backup_repo = repo._format.open(repo.bzrdir,
                                         _override_transport=backup_transport)
         self.assertDifferentRepo(repo, backup_repo)
 

=== modified file 'bzrlib/tests/per_repository/test_reconcile.py'
--- bzrlib/tests/per_repository/test_reconcile.py	2011-01-27 14:27:18 +0000
+++ bzrlib/tests/per_repository/test_reconcile.py	2011-04-02 22:51:52 +0000
@@ -49,11 +49,8 @@
     def checkNoBackupInventory(self, aBzrDir):
         """Check that there is no backup inventory in aBzrDir."""
         repo = aBzrDir.open_repository()
-        # Remote repository, and possibly others, do not have
-        # _transport.
-        if getattr(repo, '_transport', None) is not None:
-            for path in repo._transport.list_dir('.'):
-                self.assertFalse('inventory.backup' in path)
+        for path in repo.control_transport.list_dir('.'):
+            self.assertFalse('inventory.backup' in path)
 
 
 class TestsNeedingReweave(TestReconcile):
@@ -197,7 +194,7 @@
     def check_missing_was_removed(self, repo):
         if repo._reconcile_backsup_inventory:
             backed_up = False
-            for path in repo._transport.list_dir('.'):
+            for path in repo.control_transport.list_dir('.'):
                 if 'inventory.backup' in path:
                     backed_up = True
             self.assertTrue(backed_up)

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2011-01-16 01:12:01 +0000
+++ bzrlib/tests/test_osutils.py	2011-04-05 12:23:17 +0000
@@ -231,6 +231,25 @@
             self.assertFalse(osutils.is_inside_or_parent_of_any(dirs, fn))
 
 
+class TestLstat(tests.TestCaseInTempDir):
+
+    def test_lstat_matches_fstat(self):
+        # On Windows, lstat and fstat don't always agree, primarily in the
+        # 'st_ino' and 'st_dev' fields. So we force them to be '0' in our
+        # custom implementation.
+        if sys.platform == 'win32':
+            # We only have special lstat/fstat if we have the extension.
+            # Without it, we may end up re-reading content when we don't have
+            # to, but otherwise it doesn't effect correctness.
+            self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
+        f = open('test-file.txt', 'wb')
+        self.addCleanup(f.close)
+        f.write('some content\n')
+        f.flush()
+        self.assertEqualStat(osutils.fstat(f.fileno()),
+                             osutils.lstat('test-file.txt'))
+
+
 class TestRmTree(tests.TestCaseInTempDir):
 
     def test_rmtree(self):

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2011-04-04 13:37:25 +0000
+++ bzrlib/tests/test_selftest.py	2011-04-05 12:11:26 +0000
@@ -749,11 +749,6 @@
         self.check_timing(ShortDelayTestCase('test_short_delay'),
                           r"^ +[0-9]+ms$")
 
-    def _patch_get_bzr_source_tree(self):
-        # Reading from the actual source tree breaks isolation, but we don't
-        # want to assume that thats *all* that would happen.
-        self.overrideAttr(bzrlib.version, '_get_bzr_source_tree', lambda: None)
-
     def _time_hello_world_encoding(self):
         """Profile two sleep calls
 
@@ -1178,15 +1173,6 @@
             ],
             lines[-3:])
 
-    def _patch_get_bzr_source_tree(self):
-        # Reading from the actual source tree breaks isolation, but we don't
-        # want to assume that thats *all* that would happen.
-        self._get_source_tree_calls = []
-        def new_get():
-            self._get_source_tree_calls.append("called")
-            return None
-        self.overrideAttr(bzrlib.version, '_get_bzr_source_tree',  new_get)
-
     def test_verbose_test_count(self):
         """A verbose test run reports the right test count at the start"""
         suite = TestUtil.TestSuite([

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-04-04 13:38:54 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-04-05 12:34:31 +0000
@@ -44,12 +44,11 @@
 
  * Lazy hooks are now reset between test runs. (Jelmer Vernooij, #745566)
 
-* ``TreeTransform`` now calls ``_observed_sha1`` for content that it
+* ``TreeTransform.create_file`` can trigger calls to ``_observed_sha1``
+  for content that it
   creates. This means that ``merge`` and ``checkout`` should be able to
-  cache the sha values, and avoid re-reading the files on the next ``bzr
-  status`` that gets run. Further, on Windows we now properly suppress
-  ``st_dev`` and ``st_ino`` so that commit also avoids re-doing the work.
-  (John Arbash Meinel, #740932)
+  cache the sha values, and avoid re-reading the files on the next
+  ``bzr status`` that gets run. (John Arbash Meinel, #740932)
 
 Documentation
 *************
@@ -80,6 +79,11 @@
 .. Major internal changes, unlikely to be visible to users or plugin 
    developers, but interesting for bzr developers.
 
+* Added ``osutils.lstat`` and ``osutils.fstat``. These are just the ``os``
+  functions on Linux, but they are wrapped on Windows so that fstat
+  matches lstat results across all python versions.
+  (John Arbash Meinel)
+
 Testing
 *******
 

